@@ -4,8 +4,11 @@ use solana_program::program_error::ProgramError;
 
 pub use switchboard_utils::{FastRoundResultAccountData, FastRoundResult, fast_parse_switchboard_result};
 pub use switchboard_protos::protos::aggregator_state::AggregatorState;
+pub use switchboard_protos::protos::aggregator_state::mod_AggregatorState;
 pub use switchboard_protos::protos::aggregator_state::RoundResult;
 pub use switchboard_protos::protos::switchboard_account_types::SwitchboardAccountType;
+use solana_program::pubkey::Pubkey;
+use zerocopy::AsBytes;
 
 /// Returns whether the current open round is considered valid for usage.
 pub fn is_current_round_valid(aggregator: &AggregatorState) -> Result<bool, ProgramError> {
@@ -41,6 +44,42 @@ pub fn get_aggregator_result(aggregator: &AggregatorState) -> Result<RoundResult
     maybe_round.ok_or(ProgramError::InvalidAccountData)
 }
 
+pub struct BundleAccount<'a> {
+    account_info: &'a AccountInfo<'a>
+}
+
+impl<'a> BundleAccount<'a> {
+    const BUFFER_SIZE: usize = 500;
+
+    pub fn new(account: &'a AccountInfo<'a>) -> Result<Self, ProgramError> {
+        let buf = account.try_borrow_data()?;
+        if buf.len() == 0 || buf[0] != SwitchboardAccountType::TYPE_BUNDLE as u8 {
+            return Err(ProgramError::InvalidAccountData);
+        }
+        Ok(Self{
+            account_info: account
+        })
+    }
+
+    pub fn get_idx(&self, idx: usize) -> Result<FastRoundResultAccountData, ProgramError> {
+        let offset: usize = 1 + (idx * Self::BUFFER_SIZE);
+        let term = offset + std::mem::size_of::<FastRoundResultAccountData>();
+        let buf = self.account_info.try_borrow_data()?;
+        if buf.len() < term {
+            return Err(ProgramError::InvalidArgument);
+        }
+        let mut res = FastRoundResultAccountData {
+            ..Default::default()
+        };
+        let recv = res.as_bytes_mut();
+        recv.copy_from_slice(&buf[offset..term]);
+        if res.result.round_open_slot == 0 {
+            return Err(ProgramError::InvalidArgument);
+        }
+        Ok(res)
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -78,6 +117,7 @@ mod tests {
             current_round_result: Some(current_round),
             last_round_result: Some(last_round),
             parse_optimized_result_address: None,
+            bundle_auth_addresses: Vec::new(),
         }
     }
 
